from StringIO import StringIO
from Products.Archetypes import listTypes
from Products.Archetypes.Extensions.utils import installTypes, install_subskin
from Products.Maps.config import *

from Products.CMFCore.utils import getToolByName

from Products.CMFPlone.migrations.migration_util import safeEditProperty

javascripts = {
    'maps-config.js': {
        'compression': 'full',
        'expression': 'object/@@maps_googlemaps_view/enabled | nothing',
    },
    'maps-googlemaps.js': {
        'compression': 'full',
        'expression': 'object/@@maps_googlemaps_view/enabled | nothing',
    },
}

styles = {
    'maps.css': {},
}

def install(self, reinstall=False):
    out = StringIO()

    classes = listTypes(PROJECTNAME)
    installTypes(self, out, classes, PROJECTNAME)
    install_subskin(self, out, GLOBALS)

    setupPortalFactory(self, out)

    installJavascript(self, out)
    installCSS(self, out)
    installProperties(self, out, reinstall)
    installMapView(self, out)

    print >> out, 'Successfully installed %s' % PROJECTNAME

    return out.getvalue()

def installJavascript(portal, out):
    js = getToolByName(portal, 'portal_javascripts')
    ids = js.getResourceIds()
    for script_id in javascripts:
        if script_id not in ids:
            js.registerScript(script_id, **javascripts[script_id])
            print >> out, 'Installed %s.' % script_id

def installCSS(portal, out):
    css = getToolByName(portal, 'portal_css')
    ids = css.getResourceIds()
    for style_id in styles:
        if style_id not in ids:
            css.registerStylesheet(style_id, **styles[style_id])
            print >> out, 'Installed %s.' % style_id

def installProperties(self, out, reinstall=False):
    """ Add property sheet to portal_properties """

    portal_props = getToolByName(self, 'portal_properties')
    if not hasattr(portal_props, PROPERTY_SHEET):
        portal_props.addPropertySheet(PROPERTY_SHEET, 'Maps Properties')
        out.write('Added %s property sheet...\n' % PROPERTY_SHEET)
    ap = getattr(portal_props, PROPERTY_SHEET)
    map_google_api_keys = MAP_GOOGLE_API_KEYS
    if reinstall and hasattr(ap, 'map_api_keys'):
        map_google_api_keys = getattr(ap, 'map_api_keys')
        ap._delProperty('map_api_keys')
        out.write('Removed map_api_keys property.')
    if not hasattr(ap, PROPERTY_GOOGLE_KEYS_FIELD):
        safeEditProperty(ap, PROPERTY_GOOGLE_KEYS_FIELD, map_google_api_keys, 'lines')
        out.write('Added %s property field to %s property sheet...\n' % (PROPERTY_GOOGLE_KEYS_FIELD, PROPERTY_SHEET))
    if not hasattr(ap, PROPERTY_MARKERS_FIELD):
        safeEditProperty(ap, PROPERTY_MARKERS_FIELD, MAP_MARKERS, 'lines')
        out.write('Added %s property field to %s property sheet...\n' % (PROPERTY_MARKERS_FIELD, PROPERTY_SHEET))
    if not hasattr(ap, PROPERTY_DEFAULT_LOCATION_FIELD):
        safeEditProperty(ap, PROPERTY_DEFAULT_LOCATION_FIELD, "0.0, 0.0", 'string')
        out.write('Added %s property field to %s property sheet...\n' % (PROPERTY_DEFAULT_LOCATION_FIELD, PROPERTY_SHEET))
    if not hasattr(ap, PROPERTY_DEFAULT_MAPTYPE_FIELD):
        safeEditProperty(ap, PROPERTY_DEFAULT_MAPTYPE_FIELD, "normal", 'string')
        out.write('Added %s property field to %s property sheet...\n' % (PROPERTY_DEFAULT_MAPTYPE_FIELD, PROPERTY_SHEET))
    else: out.write('Skipped adding property...\n')

def installMapView(self, out):
    ttool = getToolByName(self, 'portal_types', None)
    if ttool is not None:
        for typename in ('Folder', 'Topic'):
            fti = getattr(ttool.aq_inner.aq_explicit, typename, None)
            if fti is not None:
                views = fti.getAvailableViewMethods(self)
                if 'maps_map' not in views:
                    views = views + ('maps_map',)
                    fti.manage_changeProperties(view_methods=views)
                    print >> out, "Installed Maps view for %s." % typename

def setupPortalFactory(self, out):
    # make new types use portal_factory
    ft = getToolByName(self, 'portal_factory')
    portal_factory_types = ft.getFactoryTypes().keys()
    for t in ('GeoLocation',):

        if t not in portal_factory_types:
            portal_factory_types.append(t)
            ft.manage_setPortalFactoryTypes(listOfTypeIds=portal_factory_types)

    print >> out, 'New types use portal_factory'
