
// start namespace
var mapsGoogleMaps = function() {
    // local shadows for improved packing

    _mapsConfig = mapsConfig;
    _mapsConfig_google = _mapsConfig.google;

    // privates

    _markericons = null;
    _defaultmaptype = null;

    function _initDefaults($$defaults) {
        if (_markericons == null) {
            _markericons = {};
            for (var i=0; i<$$defaults.markericons.length; i++) {
                var $definition = $$defaults.markericons[i];
                var $$icon = new GIcon();
                $$icon.image = $definition['icon'];
                $$icon.iconSize = new GSize(parseInt($definition['iconSize'][0]), parseInt($definition['iconSize'][1]));
                $$icon.iconAnchor = new GPoint(parseInt($definition['iconAnchor'][0]), parseInt($definition['iconAnchor'][1]));
                $$icon.infoWindowAnchor = new GPoint(parseInt($definition['infoWindowAnchor'][0]), parseInt($definition['infoWindowAnchor'][1]));
                $$icon.shadow = $definition['shadow'];
                $$icon.shadowSize = new GSize(parseInt($definition['shadowSize'][0]), parseInt($definition['shadowSize'][1]));
                $$icon.infoShadowAnchor = new GPoint(parseInt($definition['infoShadowAnchor'][0]), parseInt($definition['infoShadowAnchor'][1]));
                _markericons[$definition['name']] = $$icon;
            }
        }
        if (_defaultmaptype == null) {
            if ($$defaults.defaultmaptype == 'satellite') {
                _defaultmaptype = G_SATELLITE_MAP;
            } else if ($$defaults.defaultmaptype == 'hybrid') {
                _defaultmaptype = G_HYBRID_MAP;
            } else {
                _defaultmaptype = G_NORMAL_MAP;
            }
        }
    };

    function _addInfoWindow($marker, $node) {
        // this needs to be done in a seperate function to keep the correct
        // references to node and marker
        GEvent.addListener($marker, "click", function() {
            $marker.openInfoWindow($node, {maxWidth: _mapsConfig_google.maxinfowidth});
        });
    };

    function _getLocations($node) {
        var $locations = cssQuery("dl.marker", $node);
        var $result = [];

        for (var i=0; i < $locations.length; i++) {
            var $data = {};
            $node = $locations[i];
            var $geo = cssQuery(".geo", $node);
            if ($geo.length > 0) {
                $geo = $geo[0];
                var $$lat_node = cssQuery(".latitude", $geo);
                var $$long_node = cssQuery(".longitude", $geo);
                if ($$lat_node.length > 0 && $$long_node.length > 0) {
                    $data['point'] = new GLatLng(
                        parseFloat(getInnerTextFast($$lat_node[0])),
                        parseFloat(getInnerTextFast($$long_node[0]))
                    );
                } else {
                    // no point
                    continue;
                }
                $geo.parentNode.removeChild($geo);
            }
            var $$icon = cssQuery("img.marker", $node);
            if ($$icon.length > 0) {
                $$icon = $$icon[0];
                $alt = $$icon.alt;
                $$icon.parentNode.removeChild($$icon);
                $$icon = _markericons[$alt];
            }
            $data['marker'] = new GMarker($data['point'], $$icon);
            addClassName($node, "mapsMarker");
            _addInfoWindow($data['marker'], $node);
            $node.parentNode.removeChild($node);
            $result.push($data);
        }
        return $result;
    };

    function _getBounds($locations) {
        var $bounds = new GLatLngBounds();

        for (var i=0; i < $locations.length; i++) {
            $bounds.extend($locations[i]['point']);
        }
        return $bounds;
    };

    function _initMap($node) {
        $node.style.height = _mapsConfig_google.height;
        var $locations = _getLocations($node);
        var $bounds = _getBounds($locations);
        var $center = $bounds.getCenter();
        var $$map_node = document.createElement('div');
        $$map_node.style.height = _mapsConfig_google.height;
        $node.appendChild($$map_node);
        var $map = new GMap2($$map_node);
        var $zoom_level = $map.getBoundsZoomLevel($bounds);
        if ($zoom_level > _mapsConfig_google.maxzoomlevel)
            $zoom_level = _mapsConfig_google.maxzoomlevel;
        $map.setCenter($center, $zoom_level, _defaultmaptype);
        $map.addControl(new GLargeMapControl());
        if (_mapsConfig_google.selectablemaptypes) {
            $map.addControl(new GMapTypeControl());
        }
        for (var i=0; i < $locations.length; i++) {
            $map.addOverlay($locations[i]['marker']);
        }
    };

    function _setupGeocoding($input, $map, $$marker, $location) {
        var $geocoder = new GClientGeocoder();
        var $query = document.createElement('input');
        var $search = document.createElement('input');
        var $form = null;
        var $old_submit = null;

        $form = $input;
        do {
            if ($form.tagName) {
                if ($form.tagName.toLowerCase() == 'form') {
                    break;
                }
                if ($form.tagName.toLowerCase() == 'body') {
                    $form = null;
                    break;
                }
                $form = $form.parentNode;
            }
        } while ($form);

        $input.style.display = "none";
        $query.setAttribute("type", "text");
        $search.setAttribute("type", "button");
        $search.value = "Search";
        $search.className = "searchButton";

        $$func = function(e) {
            var $address = $query.value;
            $geocoder.getLatLng(
                $address,
                function($point) {
                    if (!$point) {
                        $msg = _mapsConfig_google.locationnotfound.replace(/\[LOCATION\]/, $address);
                        alert($msg);
                    } else {
                        $value = $point.lat() + ", " + $point.lng();
                        $input.value = $value;
                        $location.innerHTML = $value;
                        $$marker.setPoint($point);
                        $$marker.openInfoWindowHtml($address);
                        $map.setCenter($point, _mapsConfig_google.initialzoomlevel);
                    }
                }
            );
            return false;
        };
        $query.onfocus = function(e) {
            if ($form) {
                $old_submit = $form.onsubmit;
                $form.onsubmit = $$func;
            }
        };
        $query.onblur = function(e) {
            if ($form) {
                $form.onsubmit = $old_submit;
            }
        };
        $search.onclick = $$func;

        $input.parentNode.insertBefore($query, $input);
        $input.parentNode.insertBefore($search, $input);
    };

    function _initLocationEditMap($node) {
        var $input = cssQuery("input", $node);
        if ($input.length != 1)
            return;
        $input = $input[0];

        var $location = document.createElement('div');
        addClassName($location, "locationString discreet");

        var $$map_node = document.createElement('div');
        $$map_node.style.height = _mapsConfig_google.height;
        $node.appendChild($$map_node);
        $node.appendChild($location);

        var $map = new GMap2($$map_node);
        var $value = $input.value;
        $location.innerHTML = $value;
        $value = $value.split(',');
        var $center = new GLatLng(parseFloat($value[0]), parseFloat($value[1]));
        $map.setCenter($center, _mapsConfig_google.initialzoomlevel, _defaultmaptype);
        $map.addControl(new GLargeMapControl());
        if (_mapsConfig_google.selectablemaptypes) {
            $map.addControl(new GMapTypeControl());
        }
        var $$marker = new GMarker($center, {draggable: true});
        $map.addOverlay($$marker);
        GEvent.addListener($$marker, "dragend", function() {
            var $point = $$marker.getPoint();
            var $value = $point.lat() + ", " + $point.lng();
            $input.value = $value;
            $location.innerHTML = $value;
        });
        GEvent.addListener($map, "click", function($overlay, $point) {
            if (!$overlay) {
                $$marker.setPoint($point);
                var $value = $point.lat() + ", " + $point.lng();
                $input.value = $value;
                $location.innerHTML = $value;
            }
        });
        _setupGeocoding($input, $map, $$marker, $location);
    };

    // namespace dictionary
    return {
        init: function() {
            registerEventListener(window, 'unload', GUnload);

            if (GBrowserIsCompatible()) {
                _initDefaults(_mapsConfig_google);

                var $maps = cssQuery("div.googleMapView");
                for (var i=0; i < $maps.length; i++) {
                    _initMap($maps[i]);
                }
                var $maps = cssQuery("div.googleMapEdit");
                for (var i=0; i < $maps.length; i++) {
                    _initLocationEditMap($maps[i]);
                }
            }
        },

        loadJS: function(url) {
            document.write('<'+'script type="text/javascript" src="'+url+'"><'+'/script>');
        }

    };
// end namespace
}();

mapsGoogleMaps.loadJS("http://maps.google.com/maps?file=api&v=2&key="+mapsConfig.google.apikey);
registerEventListener(window, 'load', mapsGoogleMaps.init);
